package se.chalmers.cs.gf.tramdemo;

import java.awt.Point;
import java.util.*;

import se.chalmers.cs.gf.dialogutil.*;

public class TramInput implements TextInput {

        /** Time allowed for clicks after end of text input (in milliseconds). */
        private static final int CLICK_TIMEOUT = 1000;

        protected TextListenerList listeners = new TextListenerList();

        private MapClickEventCollector clicks;

        private List<TextInput> textInputs;

        private String text = null;

        public TramInput() {
                clicks = new MapClickEventCollector();
                textInputs = new LinkedList<TextInput>();
                new InputThread().start();
        }

        public void addTextInput(TextInput textInput) {
                textInputs.add(textInput);
                textInput.addTextListener(new TextInputListener());
        }

        public void addClickInput(TramMap map) {
                map.addMapClickListener(clicks);
        }

        public void addTextListener(TextListener l) {
                listeners.add(l);
        }

        public void removeTextListener(TextListener l) {
                listeners.remove(l);
        }

        protected void fireTextEvent(String text) {
                listeners.fireTextEvent(text);
        }

        public void setEnabled(boolean enabled) {
                // FIXME: disable click inputs too
                setTextInputsEnabled(enabled);
        }

        private void setTextInputsEnabled(boolean enabled) {
                for (TextInput i : textInputs)
                        i.setEnabled(enabled);
        }

        private synchronized void setText(String text) {
                this.text = text;
                notifyAll();
        }

        private class InputThread extends Thread {
                public void run() {
                        while (true) {
                                // wait for text input
                                String inputText;
                                synchronized (TramInput.this) {
                                        while (text == null) {
                                                try {
                                                        TramInput.this.wait();
                                                } catch (InterruptedException ex) {}
                                        }
                                        inputText = text;
                                        text = null;
                                }

                                // wait a while for more clicks
                                try {
                                        Thread.sleep(CLICK_TIMEOUT);
                                } catch (InterruptedException ex) { }

                                // get and clear clicks
                                List<MapClickEvent> inputClicks = clicks.getAndClearClicks();

                                // FIXME: remove clicks that are too old

                                // handle input
                                String queryString = makeQueryString(inputText, inputClicks);
                                fireTextEvent(queryString);

                                // re-enable all inputs
                                setEnabled(true);
                        }
                }
        }

        private static String makeQueryString(String text, List<MapClickEvent> clicks) {
                StringBuilder r = new StringBuilder(text);
                r.append(";");
                for (MapClickEvent e : clicks) {
                        r.append(" [");
                        Iterator<String> it = e.getRegions().iterator();
                        while (it.hasNext()) {
                                // FIXME: do real escaping
                                String esc = '"' + it.next() + '"';
                                r.append(esc);
                                if (it.hasNext())
                                        r.append(",");
                        }
                        r.append("]");
/*                        
                        Point p = e.getPoint();
                        r.append(" (");
                        r.append(p.x);
                        r.append(",");
                        r.append(p.y);
                        r.append(")");
*/
                }
                return r.toString();
        }

        private class TextInputListener implements TextListener {
                public void textEvent(final TextEvent e) {
                        // disable text inputs, will be re-enabled after input has
                        // been handled
                        setTextInputsEnabled(false);

                        setText(e.getText());
                }
        }

}
